@@ -13,15 +13,16 @@
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the
  *  Free Software Foundation, Inc.,
- *  59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ *  51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- *  $Id: ppdfile.h 234 2008-10-19 20:41:51Z ap2c $
+ *  $Id$
  * 
  */
 #ifndef _PPDFILE_H_
 #define _PPDFILE_H_
 
 #include <cups/ppd.h>
+#include <stdlib.h>
 
 /**
   * @class PPDFile
