@@ -7,14 +7,14 @@
 #  the content of the file where the command is located. To finish, it calls 
 #  ppdc to compile the file and generate the PPD drivers.
 #
-# $Id: compile.sh 105 2007-11-16 02:20:18Z ap2c $
+# $Id$
 # 
 
 #
 # Function parseFile
 #
 parseFile() {
-    while read LINE; do
+    while IFS= read LINE; do
         if [ -n "`echo "$LINE" | grep '^[ \t]*#import[ \t]*"[a-zA-Z0-9\.\-]*"'`" ]; then
             FILE=`echo "$LINE" | sed -re 's/[ \t]*#import[ \t]"([a-zA-Z0-9\.\-]*)"/\1/'`
             parseFile $FILE $2
@@ -28,16 +28,16 @@ parseFile() {
 #
 # Main script
 #
-if [ $2 == "debug" ]; then
-    TMPFILE="output.drv"
+if [ "$2" = "drv" ]; then
     DRIVER=$1
+    OUTFILE=${DRIVER%.in}
     shift 1
 
-    echo "" > $TMPFILE
-    parseFile $DRIVER $TMPFILE
+    echo "" > $OUTFILE
+    parseFile $DRIVER $OUTFILE
 
 
-elif [ $2 == "lang" ]; then
+elif [ "$2" = "lang" ]; then
     if [ -z $TMP ]; then
         TMP="/tmp"
     fi;
