@@ -13,9 +13,9 @@
  *   You should have received a copy of the GNU General Public License
  *   along with this program; if not, write to the
  *   Free Software Foundation, Inc.,
- *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- *   $Id: qpdl.cpp 124 2008-01-27 22:49:49Z ap2c $
+ *   $Id$
  */
 #include "qpdl.h"
 #include <QtCore/QFile>
@@ -135,9 +135,9 @@ QPDLDocument::Result QPDLDocument::_readPageHeader(QFile& data,
 
     // Printable area size
     _width = ((quint8)header.at(HEADER_WIDTH_H) << 8) + 
-        header.at(HEADER_WIDTH_L);
+        (quint8)header.at(HEADER_WIDTH_L);
     _height = ((quint8)header.at(HEADER_HEIGHT_H) << 8) + 
-        header.at(HEADER_HEIGHT_L);
+        (quint8)header.at(HEADER_HEIGHT_L);
 
     // Copies number
     _nrCopies = ((quint8)header.at(HEADER_NRCOPIES_H) << 8) +
@@ -224,11 +224,13 @@ QPDLDocument::Result QPDLDocument::_readPageContent(QFile& data,
             (quint8)header.at(BAND_WIDTH_L);
         height = ((quint8)header.at(BAND_HEIGHT_H) << 8) + 
             (quint8)header.at(BAND_HEIGHT_L);
-        if (!_quiet)
+        if (!_quiet) {
             out << '.';
+            out.flush();
+        }
 
-        // Read the band data -- SPLc in QPDL V. 0 or 1
-        if (_type == SPLc && _qpdl <= 1) {
+        // Read the band data -- SPLc in QPDL V. 0, 1 or 5
+        if (_type == SPLc && (_qpdl <= 1 || _qpdl == 5)) {
             while (!data.atEnd()) {
                 // Read the color
                 data.getChar(&color);
@@ -346,7 +348,7 @@ bool QPDLDocument::_processBandAnalysis(quint8 color, quint16 width,
                 QChar('0')) << endl;
             return false;
         }
-        content.remove(content.size() - 5, 4);
+        content.remove(content.size() - 4, 4);
     }
     content.remove(0, 4);
 
